#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "Monitor.h"
#include "UCA0_UART.h"
#include "Keyboard.h"
#include "UserEntry.h"
#include "DAC7512.h"
#include "ADC10.h"

#define MONITOR_MIN_TX_QUEUE	32
#define RAM_ADDRESS_MIN			0x0200
#define RAM_ADDRESS_MAX			0x05FF
#define INFO_ADDRESS_MIN		0x1000
#define INFO_ADDRESS_MAX		0x10FF
#define MAIN_ADDRESS_MIN		0x8000
#define MAIN_ADDRESS_MAX		0xFFFF

static __no_init SMonitorInfo const * mMonitor_ptHostInfo;
static __no_init PFN_STATE_VOID_VOID * mMonitor_ppfnHostState;
static __no_init WORD mMonitor_wIndex;
static __no_init WORD mMonitor_wAddress;
static __no_init WORD mMonitor_wLength;
static __no_init SADC10_Info mMonitor_tADC10_Info;

// States
void Monitor_State_WaitForCommand( void );
void Monitor_State_SendMenu( void );
void Monitor_State_GetUserEntryValue( void );
void Monitor_State_ShowReadMemoryResults( void );

// Helper Functions
void Monitor_Enter_WaitForCommand( void );
void Monitor_Enter_SendMenu( void );
void Monitor_StartReadMemory( void );
void Monitor_Enter_GetReadMemoryAddress( void );
void Monitor_Enter_GetReadMemoryLength( void );
void Monitor_Enter_ShowReadMemoryResults( void );
void Monitor_StartWriteMemory( void );
void Monitor_Enter_GetWriteMemoryAddress( void );
void Monitor_Enter_GetWriteMemoryValue( void );
void Monitor_StartWriteDAC( void );
void Monitor_Enter_GetValueDAC( void );
void Monitor_StartConvertADC10( void );
void Monitor_Enter_GetChannelADC10( void );
void Monitor_Enter_ConvertADC10( void );
void Monitor_ShowPrompt( void );

// Returns TRUE if the mMonitor_wAddress value contains a
// valid RAM or flash address, FALSE if not
// Call with pwValue set to NULL to simply check the address
BOOL Monitor_WriteMemory( PWORD pwValue );

// Event Sinks
void Monitor_RxEventSink( void );

// User Entry Callbacks
void Monitor_ReadMemory_OnAddressData( WORD wOut );
void Monitor_ReadMemory_OnLengthData( WORD wOut );
void Monitor_ReadMemory_OnAbandon( void );
void Monitor_ReadMemory_OnExcessInput( BYTE byIn );
void Monitor_WriteMemory_OnAddressData( WORD wOut );
void Monitor_WriteMemory_OnValueMoreData( WORD wOut );
void Monitor_WriteMemory_OnValueDoneData( WORD wOut );
void Monitor_WriteMemory_OnAbandon( void );
void Monitor_WriteMemory_OnExcessInput( BYTE byIn );
void Monitor_WriteDAC_OnValueMore( WORD wOut );
void Monitor_WriteDAC_OnValueDone( WORD wOut );
void Monitor_WriteDAC_OnAbandon( void );
void Monitor_WriteDAC_OnExcessInput( BYTE byIn );
void Monitor_ADC10_OnChannel( WORD wOut );
void Monitor_ADC10_OnAbandon( void );
void Monitor_ADC10_OnExcessInput( BYTE byIn );

// ADC10 Callback
void Monitor_ADC10_OnExit( WORD wADC10 );

// User Entry Callback Information Structures
static const SUserEntryInfo mMonitor_tReadMemoryAddressInfo =
{
	Monitor_ReadMemory_OnAddressData,
	Monitor_ReadMemory_OnAddressData,
	Monitor_ReadMemory_OnAbandon,
	Monitor_ReadMemory_OnExcessInput
};

static const SUserEntryInfo mMonitor_tReadMemoryLengthInfo =
{
	Monitor_ReadMemory_OnLengthData,
	Monitor_ReadMemory_OnLengthData,
	Monitor_ReadMemory_OnAbandon,
	Monitor_ReadMemory_OnExcessInput
};

static const SUserEntryInfo mMonitor_tWriteMemoryAddressInfo =
{
	Monitor_WriteMemory_OnAddressData,
	Monitor_WriteMemory_OnAddressData,
	Monitor_WriteMemory_OnAbandon,
	Monitor_WriteMemory_OnExcessInput
};

static const SUserEntryInfo mMonitor_tWriteMemoryValueInfo =
{
	Monitor_WriteMemory_OnValueMoreData,
	Monitor_WriteMemory_OnValueDoneData,
	Monitor_WriteMemory_OnAbandon,
	Monitor_WriteMemory_OnExcessInput
};

static const SUserEntryInfo mMonitor_tWriteDAC_Info =
{
	Monitor_WriteDAC_OnValueMore,
	Monitor_WriteDAC_OnValueDone,
	Monitor_WriteDAC_OnAbandon,
	Monitor_WriteDAC_OnExcessInput
};

static const SUserEntryInfo mMonitor_tChannelADC10_Info =
{
	Monitor_ADC10_OnChannel,
	Monitor_ADC10_OnChannel,
	Monitor_ADC10_OnAbandon,
	Monitor_ADC10_OnExcessInput
};

// Menu strings
static const char mMonitor_szMenuDAC[] = "D - DAC\n";
static const char mMonitor_szMenuQuit[] = "Q - Quit to host\n";
static const char mMonitor_szMenuReadMemory[] = "R - Read memory\n";
static const char mMonitor_szMenuWriteMemory[] = "W - Write memory\n";
static const char mMonitor_szMenuShowMenu[] = "? - Show this long menu\n";
static const char mMonitor_szMenuADC[] = "A - ADC\n";
// Add more menu strings here

// Menu array
static const PCSZ mMonitor_aszMenu[] =
{	mMonitor_szMenuADC,
	mMonitor_szMenuDAC,
	mMonitor_szMenuQuit,
	mMonitor_szMenuReadMemory,
	mMonitor_szMenuWriteMemory,
	mMonitor_szMenuShowMenu,
	// And add the string pointers here
};
static const WORD mMonitor_wMenuStringCount =
	sizeof(mMonitor_aszMenu)/sizeof(char *);

void Monitor_InitializeHW( void )
{
	DAC7512_InitializeHW();
	ADC10_InitializeHW();
}

void Monitor_InitializeApp( void )
{
	DAC7512_InitializeApp();
	ADC10_InitializeApp();
}

// Event Sinks
void Monitor_RxEventSink( void )
{
	// Request the host to resume ticks
	mMonitor_ptHostInfo->pfnCallback_OnResumeTicks();
}

void Monitor_Entry( SMonitorInfo const * ptMonitorInfo )
{
	// Register the receiver event sink
	UCA0_UART_RegisterRxEventSink( Monitor_RxEventSink );

	// Flush the receive queue
	UCA0_UART_FlushRxQueue();

	// Show the banner
	UCA0_UART_QueueTxString( "\n\nAnalog Monitor v1.0\n" );

	// Save the host information
	mMonitor_ptHostInfo = ptMonitorInfo;
	mMonitor_ppfnHostState = mMonitor_ptHostInfo->ppfnHostState;

	// Transition to waiting for commands
	Monitor_Enter_WaitForCommand();
}

void Monitor_ShowPrompt( void )
{
	UCA0_UART_QueueTxString( "\n:" );
}

// States
void Monitor_Enter_WaitForCommand( void )
{
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_WaitForCommand;

	// Show the prompt
	Monitor_ShowPrompt();
}

void Monitor_State_WaitForCommand( void )
{	WORD wRx;

	wRx = UCA0_UART_GetNextRxValueEcho();
	if( NO_RX_AVAILABLE == wRx )
	// No character received
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
	
		// Request the host to suspend ticks
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks();
		return;
	}

	// Request the host to resume ticks
	mMonitor_ptHostInfo->pfnCallback_OnResumeTicks();

	if( ('a' == wRx) || ('A' == wRx) )
	// Start the convert ADC10 command
	{
		Monitor_StartConvertADC10();
		return;
	}

	if( ('d' == wRx) || ('D' == wRx) )
	// Start the write DAC command
	{
		Monitor_StartWriteDAC();
		return;
	}
	
	if( ('q' == wRx) || ('Q' == wRx) )
	// Quit to the host application
	{
		UCA0_UART_RegisterRxEventSink( NULL );
		mMonitor_ptHostInfo->pfnCallback_OnExit();
		return;
	}

	if( ('r' == wRx) || ('R' == wRx) )
	// Start the read memory command
	{
		Monitor_StartReadMemory();
		return;
	}

	if( ('w' == wRx) || ('W' == wRx) )
	// Start the write memory command
	{
		Monitor_StartWriteMemory();
		return;
	}
	
	if( '?' == wRx )
	// Show the menu
	{
		Monitor_Enter_SendMenu();
		return;
	}
	
	// Everything else is invalid
	UCA0_UART_QueueTxString( " Invalid command, ? for help\n" );
	Monitor_ShowPrompt();
}

void Monitor_Enter_SendMenu( void )
{
	// Initialize the string index
	mMonitor_wIndex = 0;
	
	// Send the command title
	UCA0_UART_QueueTxString( "\nSupported commands:\n\n" );
	
	// Transition states
	*mMonitor_ppfnHostState = Monitor_State_SendMenu;
}

void Monitor_State_SendMenu( void )
{	WORD wAvailableTxQueue;

	// Check whether room is available for a new string
	wAvailableTxQueue = UCA0_UART_GetAvailableTxQueue();
	if( wAvailableTxQueue < MONITOR_MIN_TX_QUEUE )
	// Not enough room yet
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
		return;
	}
	
	// Check whether done with menu
	if( mMonitor_wIndex >= mMonitor_wMenuStringCount )
	// Sent the last string already
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Send the next string
	UCA0_UART_QueueTxString( mMonitor_aszMenu[mMonitor_wIndex++] );
}

void Monitor_StartReadMemory( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Read memory\n" );

	// Transition state
	Monitor_Enter_GetReadMemoryAddress();
}

void Monitor_Enter_GetReadMemoryAddress( void )
{
	// Print an address prompt
	UCA0_UART_QueueTxString( "Addr: 0x" );

	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tReadMemoryAddressInfo );
}

void Monitor_State_GetUserEntryValue( void )
{	WORD wRx;

	wRx = UCA0_UART_GetNextRxValueEcho();
	if( NO_RX_AVAILABLE == wRx )
	// No character received
	{
		// Call host idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
	
		// Request the host to suspend ticks
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks();
		return;
	}

	// Tick the user entry feature
	UserEntry_ConsumeBYTE( wRx );
}

void Monitor_Enter_GetReadMemoryLength( void )
{
	// Print a length prompt
	UCA0_UART_QueueTxString( "\nLength: 0x" );

	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tReadMemoryLengthInfo );
}

void Monitor_Enter_ShowReadMemoryResults( void )
{
	// Initialize the index to force an initial address display
	mMonitor_wIndex = 8;

	// Print a message
	UCA0_UART_QueueTxString( "\nReading memory:" );
	
	// Transition states
	*mMonitor_ppfnHostState = Monitor_State_ShowReadMemoryResults;
}

void Monitor_State_ShowReadMemoryResults( void )
{	WORD wAvailableTxQueue;
	WORD wThisMemoryValue;

	// Check whether room is available for a new string
	wAvailableTxQueue = UCA0_UART_GetAvailableTxQueue();
	if( wAvailableTxQueue < MONITOR_MIN_TX_QUEUE )
	// Not enough room yet
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
		return;
	}
	
	// Check whether done with displaying values
	if( 0 == mMonitor_wLength )
	// Sent the last value already
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Decide whether to create a new line and print address
	if( 8 == mMonitor_wIndex )
	// Print a new line and address
	{	
		// Print a newline plus 0x
		UCA0_UART_QueueTxString( "\n0x" );
		
		// Print the address and a colon plus space
		UCA0_UART_QueueTxHexWORD( mMonitor_wAddress );
		UCA0_UART_QueueTxString( ": " );
		
		// Reset address display index
		mMonitor_wIndex = 1;
	}
	else
	// Just another value
	{
		// Print a space
		UCA0_UART_QueueTxChar( ' ' );
		
		// and increment index
		mMonitor_wIndex++;
	}
	
	// Get the memory value
	wThisMemoryValue = *((WORD*)mMonitor_wAddress);
	
	// Print it
	UCA0_UART_QueueTxHexWORD( wThisMemoryValue );
	
	// Update the count
	mMonitor_wLength--;
	
	// Check whether address at end of 64k
	if( mMonitor_wAddress >= 0xFFFE )
	// Need to exit
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
	}
	else
	// Update address
	{	
		mMonitor_wAddress += 2;
	}
}

// Read Memory User Entry Callbacks
void Monitor_ReadMemory_OnAddressData( WORD wOut )
{
	// Check for low addresses
	if( wOut < 0x0100 )
	// Address too low
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be above 0x0100\n" );

		// Revert to collecting the address
		Monitor_Enter_GetReadMemoryAddress();
		return;
	}

	// Check for odd addresses
	if( wOut & 0x0001 )
	// Address odd
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be even\n" );

		// Revert to collecting the address
		Monitor_Enter_GetReadMemoryAddress();
		return;
	}

	// Store the parsed value
	mMonitor_wAddress = wOut;
	
	// Transition to the next state
	Monitor_Enter_GetReadMemoryLength();
}

void Monitor_ReadMemory_OnLengthData( WORD wOut )
{
	// Store the parsed value
	mMonitor_wLength = wOut;
	
	// Transition to the next state
	Monitor_Enter_ShowReadMemoryResults();
}

void Monitor_ReadMemory_OnAbandon( void )
{
	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_ReadMemory_OnExcessInput( BYTE byIn )
{
	// Print a warning
	UCA0_UART_QueueTxString( "\nExcess input warning!\n" );

	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_StartWriteMemory( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Write memory\n" );

	// Transition state
	Monitor_Enter_GetWriteMemoryAddress();
}

void Monitor_Enter_GetWriteMemoryAddress( void )
{
	// Print an address prompt
	UCA0_UART_QueueTxString( "Addr: 0x" );

	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tWriteMemoryAddressInfo );
}

void Monitor_Enter_GetWriteMemoryValue( void )
{
	// Print the address as a prompt
	UCA0_UART_QueueTxString( "\n" );
	UCA0_UART_QueueTxHexWORD( mMonitor_wAddress );
	UCA0_UART_QueueTxString( ": 0x" );
	
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tWriteMemoryValueInfo );
}

void Monitor_WriteMemory_OnAddressData( WORD wOut )
{	BOOL bResult;

	// Check for odd addresses
	if( wOut & 0x0001 )
	// Address odd
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be even\n" );

		// Revert to collecting the address
		Monitor_Enter_GetWriteMemoryAddress();
		return;
	}

	// Store the parsed value
	mMonitor_wAddress = wOut;

	// Check for a valid address without writing anything
	bResult = Monitor_WriteMemory( NULL );
	if( FALSE == bResult )
	// Address invalid
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be in RAM or flash\n" );

		// Revert to collecting the address
		Monitor_Enter_GetWriteMemoryAddress();
		return;
	}
	
	// Transition to the next state
	Monitor_Enter_GetWriteMemoryValue();
}

void Monitor_WriteMemory_OnValueMoreData( WORD wOut )
{	BOOL bResult;

	// Attempt to write the value
	bResult = Monitor_WriteMemory( &wOut );
	if( FALSE == bResult )
	// Address is invalid, stop process
	{
		// Revert to waiting for a new command
		Monitor_Enter_WaitForCommand();
		return;
	}
	
	// Make sure address isn't about to overflow
	if( mMonitor_wAddress >= 0xFFFE )
	{
		// Revert to waiting for a new command
		Monitor_Enter_WaitForCommand();
		return;
	}
	
	// Update the address
	mMonitor_wAddress += 2;
	
	// Check it
	bResult = Monitor_WriteMemory( NULL );
	if( FALSE == bResult )
	// Next address is invalid, stop process
	{
		// Revert to waiting for a new command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Go get another value for the new address
	Monitor_Enter_GetWriteMemoryValue();
}

void Monitor_WriteMemory_OnValueDoneData( WORD wOut )
{
	// Attempt to write the value
	Monitor_WriteMemory( &wOut );

	// Unconditionally revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_WriteMemory_OnAbandon( void )
{
	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_WriteMemory_OnExcessInput( BYTE byIn )
{
	// Print a warning
	UCA0_UART_QueueTxString( "\nExcess input warning!\n" );

	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

BOOL Monitor_WriteMemory( PWORD pwValue )
// Returns TRUE if the mMonitor_wAddress value contains a
// valid RAM or flash address, FALSE if not
// Call with pwValue set to NULL to simply check the address
{
	if( mMonitor_wAddress < RAM_ADDRESS_MIN )
	// Address is below RAM
	{	return FALSE;
	}
	
	if( mMonitor_wAddress <= RAM_ADDRESS_MAX )
	// Address is in RAM, perform the write
	{	
		if( NULL != pwValue )
		// A value was provided, write to RAM
		{
			*((WORD*)mMonitor_wAddress) = *pwValue;
		}
		return TRUE;
	}

	if( mMonitor_wAddress < INFO_ADDRESS_MIN )
	// Address is below info flash
	{	return FALSE;
	}
	
	if( mMonitor_wAddress < MAIN_ADDRESS_MIN )
	// Address is below main flash
	{
		if( mMonitor_wAddress > INFO_ADDRESS_MAX )
		// Address is above info flash
		{
			return FALSE;
		}
	}
	
	// Address is either in main flash or in info flash
	if( NULL == pwValue )
	// A value was not provided
	{	return TRUE;
	}
	
	// Implement a flash write
	// Assumes SMCLK is approximately 1.2 MHz

	// Block other threads from writing the queue
	istate_t isInterruptState = __get_interrupt_state();
	__disable_interrupt();

	// Divide SMCLK by 3
	FCTL2 = (FWKEY|FSSEL1|FN1);
	// Clear LOCK
	FCTL3 = (FWKEY);
	// Enable write
	FCTL1 = (FWKEY|WRT);
	
	// Execute the write
	*((WORD*)mMonitor_wAddress) = *pwValue;
	// Code resumes here after write

	// Disable write
	FCTL1 = (FWKEY);
	// Set LOCK
	FCTL3 = (FWKEY|LOCK);
	
	// Unblock access
	__set_interrupt_state( isInterruptState );

	return TRUE;
}

void Monitor_StartWriteDAC( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Write DAC" );

	// Transition state
	Monitor_Enter_GetValueDAC();
}

void Monitor_Enter_GetValueDAC( void )
{
	// Print the address as a prompt
	UCA0_UART_QueueTxString( "\nDAC: 0x" );
	
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tWriteDAC_Info );
}

void Monitor_WriteDAC_OnValueMore( WORD wOut )
{
	// Write the value
	DAC7512_SetValue( wOut );

	// Go get another value
	Monitor_Enter_GetValueDAC();
}

void Monitor_WriteDAC_OnValueDone( WORD wOut )
{
	// Write the value
	DAC7512_SetValue( wOut );

	// Unconditionally revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_WriteDAC_OnAbandon( void )
{
	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_WriteDAC_OnExcessInput( BYTE byIn )
{
	// Print a warning
	UCA0_UART_QueueTxString( "\nExcess input warning!\n" );

	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_StartConvertADC10( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Convert ADC10" );

	// Transition state
	Monitor_Enter_GetChannelADC10();
}

void Monitor_Enter_GetChannelADC10( void )
{
	// Print the address as a prompt
	UCA0_UART_QueueTxString( "\nCh: 0x" );
	
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the BYTE user entry feature
	UserEntry_EntryBYTE( &mMonitor_tChannelADC10_Info );
}

void Monitor_Enter_ConvertADC10( void )
{
	// Prepare the ADC10 information structure
	mMonitor_tADC10_Info.pfnCallback_OnIdle =
		mMonitor_ptHostInfo->pfnCallback_OnIdle;

	mMonitor_tADC10_Info.ppfnHostState =
		mMonitor_ptHostInfo->ppfnHostState;

	mMonitor_tADC10_Info.pfnCallback_OnExit =
		Monitor_ADC10_OnExit;

	mMonitor_tADC10_Info.pfnCallback_OnSuspendTicks =
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks;

	mMonitor_tADC10_Info.pfnCallback_OnResumeTicks =
		mMonitor_ptHostInfo->pfnCallback_OnResumeTicks;

	// Launch the ADC10 mini-app
	ADC10_Entry( &mMonitor_tADC10_Info, (BYTE)mMonitor_wAddress );
}

void Monitor_ADC10_OnChannel( WORD wOut )
// User has entered a BYTE value
{
	// Make sure value is no greater than 0x0F
	if( wOut > 0x0F )
	// Invalid address
	{
		// Print a channel prompt
		UCA0_UART_QueueTxString( "\nChannel must be 0x00 to 0x0F\n" );

		// Revert to collecting another channel
		Monitor_Enter_GetChannelADC10();
		return;
	}

	// Store the parsed value
	mMonitor_wAddress = wOut;

	// Transition to the next state
	Monitor_Enter_ConvertADC10();
}

void Monitor_ADC10_OnAbandon( void )
{
	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_ADC10_OnExcessInput( BYTE byIn )
{
	// Print a warning
	UCA0_UART_QueueTxString( "\nExcess input warning!\n" );

	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_ADC10_OnExit( WORD wADC10 )
{
	// Print the value
	UCA0_UART_QueueTxString( " = 0x" );
	UCA0_UART_QueueTxHexWORD( wADC10 );

	// Revert to collecting another channel
	Monitor_Enter_GetChannelADC10();
}
