#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "Switches.h"

#define SWITCHES_TICK_DIVISOR			64

// This is the major portion of the pre-charge delay for the switch pull-ups
// Add 11 cycles to the delay defined below.  The total delay needs to allow
// five microseconds at whatever MCLK is used for the project
#define SWITCH_PULLUP_PRECHARGE_CYCLES			2

// 5 seconds of wait-for-high counts before declaring a stuck switch
#define SWITCHES_STUCK_COUNT			(5*32768/SWITCHES_TICK_DIVISOR)

// 1 second of long delay ACLKs before rechecking a stuck switch
#define SWITCHES_LONG_DELAY				(32768-SWITCHES_TICK_DIVISOR)

static __no_init PFN_STATE_VOID_VOID mSwitches_pfnCurrentState;
static __no_init PFN_STATE_VOID_VOID mSwitches_pfnNextStateAfterDelay;
static __no_init WORD mSwitches_wFailedHighCount;

void Switches_DeclareEvent( void );
void Switches_DeclareStuckEvent( void );

// States
void Switches_State_WaitForLow( void );
void Switches_State_CheckLow( void );
void Switches_State_WaitForHigh( void );
void Switches_State_CheckHigh( void );
void Switches_State_Delay( void );
void Switches_State_LongDelay( void );
void Switches_State_CheckStuck( void );

// Helper functions
void Switches_Enter_WaitForLow( void );
void Switches_Enter_WaitForHigh_InitFailureCount( void );
void Switches_Enter_WaitForHigh( void );
void Switches_Enter_Delay( PFN_STATE_VOID_VOID pfnNextState );
void Switches_Enter_LongDelay( void );
void Switches_ReEnter_WaitForHighDelay( void );
void Switches_DetectedLow( void );
void Switches_PrechargeDelay( void );

void Switches_InitializeHW( void )
{
	// Note:  Assumes TimerA3 in Continuous mode from ACLK

	// Prepare switch pullups
	P1REN |= (BIT7|BIT6);
	P1OUT |= (BIT7|BIT6);
}

void Switches_InitializeApp( void )
{
	Switches_Enter_WaitForLow();
}

void Switches_DeclareEvent( void )
{
	// Toggle P1.3
	P1OUT ^= BIT3;
}

void Switches_DeclareStuckEvent( void )
{
}

void Switches_Enter_WaitForLow( void )
{
	// Enable the switch pullup
	P1REN |= (BIT6);

	// Disable TACCR1 interrupt
	TACCTL1	&= ~(CCIE);

	// Prepare for falling edges
	P1IES |= (BIT6);

	// Clear the interrupt flag
	P1IFG &= ~(BIT6);
	
	// Enable P1.6 interrupts
	P1IE |= (BIT6);

	// Transition to the place-holder state
	mSwitches_pfnCurrentState = Switches_State_WaitForLow;
}

void Switches_State_WaitForLow( void )
{
	// This state remains as a placeholder
	// The functionality is split between the P1.6 ISR and
	// the Switches_DetectedLow() function, below.
}

void Switches_DetectedLow( void )
{
	// Disable the switch pullup
	P1REN &= ~(BIT6);

	// Cancel further P1.6 interrupts
	P1IE &= ~(BIT6);	

	// Initialize the next wakeup
	TACCR1 = TAR + SWITCHES_TICK_DIVISOR;
	
	// Enable TACCR1 interrupt
	TACCTL1 |= (CCIE);
	
	// Pin is low, delay a tick before validating
	Switches_Enter_Delay( Switches_State_CheckLow );
}

void Switches_State_CheckLow( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (BIT6);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();

	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(BIT6);

	if( (byP1IN & (BIT6)) )
	// Pin is high, restart state chain
	{	Switches_Enter_WaitForLow();
		return;
	}
	
	// Pin is low, declare the event
	Switches_DeclareEvent();
	
	// And transition
	Switches_Enter_WaitForHigh_InitFailureCount();
}

void Switches_Enter_WaitForHigh_InitFailureCount( void )
{
	// Initialize the failure count for fresh entries
	mSwitches_wFailedHighCount = SWITCHES_STUCK_COUNT;

	// Call through to the core entry logic
	Switches_Enter_WaitForHigh();
}

void Switches_Enter_WaitForHigh( void )
{
	mSwitches_pfnCurrentState = Switches_State_WaitForHigh;
}

void Switches_State_WaitForHigh( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (BIT6);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(BIT6);

	if( !(byP1IN & (BIT6)) )
	// Pin is low
	{	// Update failure count
		mSwitches_wFailedHighCount--;
		if( mSwitches_wFailedHighCount )
		// Count not expired, stay in this stream
		{	return;
		}

		// Count expired, handle as stuck
		Switches_DeclareStuckEvent();
		Switches_Enter_LongDelay();
		return;
	}
	
	// Pin is high, delay a tick before validating
	Switches_Enter_Delay( Switches_State_CheckHigh );
}

void Switches_State_CheckHigh( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (BIT6);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(BIT6);

	if( !(byP1IN & (BIT6)) )
	// Pin is low, revert to waiting for high
	{	Switches_Enter_WaitForHigh();
		return;
	}
	
	// Restart state chain
	Switches_Enter_WaitForLow();
}

void Switches_Enter_Delay( PFN_STATE_VOID_VOID pfnNextState )
{
	mSwitches_pfnNextStateAfterDelay = pfnNextState;
	mSwitches_pfnCurrentState = Switches_State_Delay;
}

void Switches_State_Delay( void )
{
	mSwitches_pfnCurrentState = mSwitches_pfnNextStateAfterDelay;
}

void Switches_Enter_LongDelay( void )
{
	TACCR1 += SWITCHES_LONG_DELAY;
	mSwitches_pfnCurrentState = Switches_State_LongDelay;
}

void Switches_State_LongDelay( void )
{
	mSwitches_pfnCurrentState = Switches_State_CheckStuck;
}

void Switches_State_CheckStuck( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (BIT6);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(BIT6);

	if( !(byP1IN & (BIT6)) )
	// Pin is low, revert to long delay
	{	Switches_Enter_LongDelay();
		return;
	}
	
	// Pin is high, attempt re-entry
	Switches_ReEnter_WaitForHighDelay();
}

void Switches_ReEnter_WaitForHighDelay( void )
{
	// Initialize the failure count for re-entries
	mSwitches_wFailedHighCount = 1;

	// Transition
	Switches_Enter_Delay( Switches_State_CheckHigh );
}

#pragma diag_suppress=Go004
#pragma optimize=size no_inline
void Switches_PrechargeDelay( void )
{
	// Delay a fixed number of cycles
	__delay_cycles( SWITCH_PULLUP_PRECHARGE_CYCLES );
}

#pragma vector=TIMERA1_VECTOR
__interrupt void TimerA3_ISR( void )
// Shared interrupt vector for TAR, CCR1 and CCR2
{
	switch ( __even_in_range(TAIV, 10) )
	{
		case  2:
		// TACCR1
			TACCR1 += (SWITCHES_TICK_DIVISOR);
			mSwitches_pfnCurrentState();
			break;
		
		case  4:
		// TACCR2
			break;
			
		case 10:
		// TAIFG
			break;
	}
}

#pragma vector=PORT1_VECTOR
__interrupt void Port1_ISR( void )
// Shared interrupt vector for all Port1 pins
{
	// Test whether P1.6 was triggered
	if( P1IFG & BIT6 )
	// P1.6 experienced an interrupt
	{		
		// Take action
		Switches_DetectedLow();
		return;
	}
}


