#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "Monitor.h"
#include "UCA0_UART.h"
#include "Keyboard.h"
#include "UserEntry.h"

#define MONITOR_MIN_TX_QUEUE	32

static __no_init SMonitorInfo const * mMonitor_ptHostInfo;
static __no_init PFN_STATE_VOID_VOID * mMonitor_ppfnHostState;
static __no_init WORD mMonitor_wIndex;
static __no_init WORD mMonitor_wAddress;
static __no_init WORD mMonitor_wLength;

// States
void Monitor_State_WaitForCommand( void );
void Monitor_State_SendMenu( void );
void Monitor_State_GetUserEntryValue( void );
void Monitor_State_ShowReadMemoryResults( void );
void Monitor_State_StartWriteMemory( void );

// Helper Functions
void Monitor_Enter_WaitForCommand( void );
void Monitor_Enter_SendMenu( void );
void Monitor_StartReadMemory( void );
void Monitor_Enter_GetReadMemoryAddress( void );
void Monitor_Enter_GetReadMemoryLength( void );
void Monitor_Enter_ShowReadMemoryResults( void );
void Monitor_Enter_StartWriteMemory( void );
void Monitor_ShowPrompt( void );

// Event Sinks
void Monitor_RxEventSink( void );

// User Entry Callbacks
void Monitor_ReadMemory_OnAddressData( WORD wOut );
void Monitor_ReadMemory_OnLengthData( WORD wOut );
void Monitor_ReadMemory_OnAbandon( void );
void Monitor_ReadMemory_OnExcessInput( BYTE byIn );

// User Entry Callback Information Structures
static const SUserEntryInfo mMonitor_tReadMemoryAddressInfo =
{
	Monitor_ReadMemory_OnAddressData,
	Monitor_ReadMemory_OnAddressData,
	Monitor_ReadMemory_OnAbandon,
	Monitor_ReadMemory_OnExcessInput
};

static const SUserEntryInfo mMonitor_tReadMemoryLengthInfo =
{
	Monitor_ReadMemory_OnLengthData,
	Monitor_ReadMemory_OnLengthData,
	Monitor_ReadMemory_OnAbandon,
	Monitor_ReadMemory_OnExcessInput
};

// Menu strings
static const char mMonitor_szMenu1[] = "Q - Quit to host\n";
static const char mMonitor_szMenu2[] = "R - Read memory\n";
static const char mMonitor_szMenu3[] = "W - Write memory\n";
static const char mMonitor_szMenu4[] = "? - Show this long menu\n";
// Add more menu strings here

// Menu array
static const PCSZ mMonitor_aszMenu[] =
{	mMonitor_szMenu1,
	mMonitor_szMenu2,
	mMonitor_szMenu3,
	mMonitor_szMenu4,
	// And add the string pointers here
};
static const WORD mMonitor_wMenuStringCount =
	sizeof(mMonitor_aszMenu)/sizeof(char *);

void Monitor_InitializeHW( void )
{
}

void Monitor_InitializeApp( void )
{
}

// Event Sinks
void Monitor_RxEventSink( void )
{
	// Request the host to resume ticks
	mMonitor_ptHostInfo->pfnCallback_OnResumeTicks();
}

void Monitor_Entry( SMonitorInfo const * ptMonitorInfo )
{
	// Register the receiver event sink
	UCA0_UART_RegisterRxEventSink( Monitor_RxEventSink );

	// Flush the receive queue
	UCA0_UART_FlushRxQueue();

	// Show the banner
	UCA0_UART_QueueTxString( "\n\nMore Monitor v1.0\n" );

	// Save the host information
	mMonitor_ptHostInfo = ptMonitorInfo;
	mMonitor_ppfnHostState = mMonitor_ptHostInfo->ppfnHostState;

	// Transition to waiting for commands
	Monitor_Enter_WaitForCommand();
}

void Monitor_ShowPrompt( void )
{
	UCA0_UART_QueueTxString( "\n:" );
}

// States
void Monitor_Enter_WaitForCommand( void )
{
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_WaitForCommand;

	// Show the prompt
	Monitor_ShowPrompt();
}

void Monitor_State_WaitForCommand( void )
{	WORD wRx;

	wRx = UCA0_UART_GetNextRxValueEcho();
	if( NO_RX_AVAILABLE == wRx )
	// No character received
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
	
		// Request the host to suspend ticks
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks();
		return;
	}

	// Request the host to resume ticks
	mMonitor_ptHostInfo->pfnCallback_OnResumeTicks();
	
	if( ('q' == wRx) || ('Q' == wRx) )
	// Quit to the host application
	{
		UCA0_UART_RegisterRxEventSink( NULL );
		mMonitor_ptHostInfo->pfnCallback_OnExit();
		return;
	}

	if( ('r' == wRx) || ('R' == wRx) )
	// Start the read memory command
	{
		Monitor_StartReadMemory();
		return;
	}

	if( ('w' == wRx) || ('W' == wRx) )
	// Start the write memory command
	{
		Monitor_Enter_StartWriteMemory();
		return;
	}
	
	if( '?' == wRx )
	// Show the menu
	{
		Monitor_Enter_SendMenu();
		return;
	}
	
	// Everything else is invalid
	UCA0_UART_QueueTxString( " Invalid command, ? for help\n" );
	Monitor_ShowPrompt();
}

void Monitor_Enter_SendMenu( void )
{
	// Initialize the string index
	mMonitor_wIndex = 0;
	
	// Send the command title
	UCA0_UART_QueueTxString( "\nSupported commands:\n\n" );
	
	// Transition states
	*mMonitor_ppfnHostState = Monitor_State_SendMenu;
}

void Monitor_State_SendMenu( void )
{	WORD wAvailableTxQueue;

	// Check whether room is available for a new string
	wAvailableTxQueue = UCA0_UART_GetAvailableTxQueue();
	if( wAvailableTxQueue < MONITOR_MIN_TX_QUEUE )
	// Not enough room yet
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
		return;
	}
	
	// Check whether done with menu
	if( mMonitor_wIndex >= mMonitor_wMenuStringCount )
	// Sent the last string already
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Send the next string
	UCA0_UART_QueueTxString( mMonitor_aszMenu[mMonitor_wIndex++] );
}

void Monitor_StartReadMemory( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Read memory\n" );

	// Transition state
	Monitor_Enter_GetReadMemoryAddress();
}

void Monitor_Enter_GetReadMemoryAddress( void )
{
	// Print an address prompt
	UCA0_UART_QueueTxString( "Addr: 0x" );

	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tReadMemoryAddressInfo );
}

void Monitor_State_GetUserEntryValue( void )
{	WORD wRx;

	wRx = UCA0_UART_GetNextRxValueEcho();
	if( NO_RX_AVAILABLE == wRx )
	// No character received
	{
		// Call host idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
	
		// Request the host to suspend ticks
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks();
		return;
	}

	// Tick the user entry feature
	UserEntry_ConsumeBYTE( wRx );
}

void Monitor_Enter_GetReadMemoryLength( void )
{
	// Print a length prompt
	UCA0_UART_QueueTxString( "\nLength: 0x" );

	// Transition
	*mMonitor_ppfnHostState = Monitor_State_GetUserEntryValue;

	// Launch the WORD user entry feature
	UserEntry_EntryWORD( &mMonitor_tReadMemoryLengthInfo );
}

void Monitor_Enter_ShowReadMemoryResults( void )
{
	// Initialize the index to force an initial address display
	mMonitor_wIndex = 8;

	// Print a message
	UCA0_UART_QueueTxString( "\nReading memory:" );
	
	// Transition states
	*mMonitor_ppfnHostState = Monitor_State_ShowReadMemoryResults;
}

void Monitor_State_ShowReadMemoryResults( void )
{	WORD wAvailableTxQueue;
	WORD wThisMemoryValue;

	// Check whether room is available for a new string
	wAvailableTxQueue = UCA0_UART_GetAvailableTxQueue();
	if( wAvailableTxQueue < MONITOR_MIN_TX_QUEUE )
	// Not enough room yet
	{
		// Call idle callback
		mMonitor_ptHostInfo->pfnCallback_OnIdle();
		return;
	}
	
	// Check whether done with displaying values
	if( 0 == mMonitor_wLength )
	// Sent the last value already
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Decide whether to create a new line and print address
	if( 8 == mMonitor_wIndex )
	// Print a new line and address
	{	
		// Print a newline plus 0x
		UCA0_UART_QueueTxString( "\n0x" );
		
		// Print the address and a colon plus space
		UCA0_UART_QueueTxHexWORD( mMonitor_wAddress );
		UCA0_UART_QueueTxString( ": " );
		
		// Reset address display index
		mMonitor_wIndex = 1;
	}
	else
	// Just another value
	{
		// Print a space
		UCA0_UART_QueueTxChar( ' ' );
		
		// and increment index
		mMonitor_wIndex++;
	}
	
	// Get the memory value
	wThisMemoryValue = *((WORD*)mMonitor_wAddress);
	
	// Print it
	UCA0_UART_QueueTxHexWORD( wThisMemoryValue );
	
	// Update the count
	mMonitor_wLength--;
	
	// Check whether address at end of 64k
	if( mMonitor_wAddress >= 0xFFFE )
	// Need to exit
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
	}
	else
	// Update address
	{	
		mMonitor_wAddress += 2;
	}
}

// Read Memory User Entry Callbacks
void Monitor_ReadMemory_OnAddressData( WORD wOut )
{
	// Check for low addresses
	if( wOut < 0x0100 )
	// Address too low
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be above 0x0100\n" );

		// Revert to collecting the address
		Monitor_Enter_GetReadMemoryAddress();
		return;
	}

	// Check for odd addresses
	if( wOut & 0x0001 )
	// Address odd
	{
		// Print an address prompt
		UCA0_UART_QueueTxString( "\nAddress must be even\n" );

		// Revert to collecting the address
		Monitor_Enter_GetReadMemoryAddress();
		return;
	}

	// Store the parsed value
	mMonitor_wAddress = wOut;
	
	// Transition to the next state
	Monitor_Enter_GetReadMemoryLength();
}

void Monitor_ReadMemory_OnLengthData( WORD wOut )
{
	// Store the parsed value
	mMonitor_wLength = wOut;
	
	// Transition to the next state
	Monitor_Enter_ShowReadMemoryResults();
}

void Monitor_ReadMemory_OnAbandon( void )
{
	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_ReadMemory_OnExcessInput( BYTE byIn )
{
	// Print a warning
	UCA0_UART_QueueTxString( "\nExcess input warning!\n" );

	// Revert to waiting for a new command
	Monitor_Enter_WaitForCommand();
}

void Monitor_Enter_StartWriteMemory( void )
{
	// Send the command title
	UCA0_UART_QueueTxString( " Write memory\n" );

	// Transition state
	*mMonitor_ppfnHostState = Monitor_State_StartWriteMemory;
}

void Monitor_State_StartWriteMemory( void )
{
	// For now, simply transition back
	Monitor_Enter_WaitForCommand();
}

