#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "Switches.h"

#define VOTE_BIT_SW1					BIT0
#define VOTE_BIT_SW2					BIT1

#define SWITCHES_TICK_DIVISOR			64

// This is the major portion of the pre-charge delay for the switch pull-ups
// Add 11 cycles to the delay defined below.  The total delay needs to allow
// five microseconds at whatever MCLK is used for the project
#define SWITCH_PULLUP_PRECHARGE_CYCLES			2

// 5 seconds of wait-for-high counts before declaring a stuck switch
#define SWITCHES_STUCK_COUNT			(5*32768/SWITCHES_TICK_DIVISOR)

// 1 second of long delay ACLKs before rechecking a stuck switch
#define SWITCHES_LONG_DELAY				32768

typedef struct StructSwitchInfo
// Defines parameters required to implement a switch
{	
	// The current state to be executed for this switch at the next tick.
	PFN_STATE_VOID_VOID pfnCurrentState;
	
	// The state to execute after the delay for this switch
	PFN_STATE_VOID_VOID pfnNextStateAfterDelay;
	
	// The number of times this switch has failed the high count
	WORD wFailedHighCount;

	// Defines the port pin to reference for this switch
	BYTE byPortPin;

	// Maintains the operating mode vote bit
	BYTE byVoteBit;
	
} SSwitchInfo;

static __no_init SSwitchInfo* mSwitches_ptCurrentSwitch;
static __no_init SSwitchInfo mSwitches_tSW1;
static __no_init SSwitchInfo mSwitches_tSW2;

static __no_init BYTE mSwitches_byTickingVotes;
static __no_init BYTE mSwitches_byStuckVotes;
static __no_init WORD mSwitches_wTickInterval;

static __no_init PFN_STATE_VOID_VOID mSwitches_pfnOperatingMode;

void Switches_DeclareEvent( void );
void Switches_DeclareStuckEvent( void );

// States
void Switches_State_WaitForLow( void );
void Switches_State_CheckLow( void );
void Switches_State_WaitForHigh( void );
void Switches_State_CheckHigh( void );
void Switches_State_Delay( void );
void Switches_State_CheckStuck( void );

// Modes
void Switches_Mode_Sleeping( void );
void Switches_Mode_Ticking( void );
void Switches_Mode_Stuck( void );

// Helper functions
void Switches_Enter_WaitForLow( void );
void Switches_Enter_WaitForHigh_InitFailureCount( void );
void Switches_Enter_WaitForHigh( void );
void Switches_Enter_Delay( PFN_STATE_VOID_VOID pfnNextState );
void Switches_Enter_CheckStuck( void );
void Switches_Exit_CheckStuck( void );
void Switches_DetectedLow( void );
void Switches_PrechargeDelay( void );

void Switches_InitializeHW( void )
{
	// Note:  Assumes TimerA3 in Continuous mode from ACLK

	// Prepare switch pullups
	P1REN |= (BIT7|BIT6);
	P1OUT |= (BIT7|BIT6);
}

void Switches_InitializeApp( void )
{
	// Set initial operating mode
	mSwitches_pfnOperatingMode = Switches_Mode_Sleeping;

	// Set to default
	mSwitches_wTickInterval = SWITCHES_TICK_DIVISOR;

	mSwitches_byTickingVotes = 0x00;
	mSwitches_byStuckVotes = 0x00;

	mSwitches_tSW1.byPortPin = BIT6;
	mSwitches_tSW1.byVoteBit = VOTE_BIT_SW1;

	mSwitches_tSW2.byPortPin = BIT7;
	mSwitches_tSW2.byVoteBit = VOTE_BIT_SW2;

	mSwitches_ptCurrentSwitch = &mSwitches_tSW1;
	Switches_Enter_WaitForLow();

	mSwitches_ptCurrentSwitch = &mSwitches_tSW2;
	Switches_Enter_WaitForLow();
}

void Switches_DeclareEvent( void )
{
	// Toggle P1.3
	P1OUT ^= BIT3;
}

void Switches_DeclareStuckEvent( void )
{
}

void Switches_Mode_Sleeping( void )
{
	if( mSwitches_byTickingVotes )
	// A switch wishes to tick
	{
		// Prepare the short interval
		mSwitches_wTickInterval = SWITCHES_TICK_DIVISOR;

		// Initialize the next wakeup
		TACCR1 = TAR + SWITCHES_TICK_DIVISOR;

		// Enable TACCR1 interrupt
		TACCTL1 |= (CCIE);
		
		// Transition to the ticking mode
		mSwitches_pfnOperatingMode = Switches_Mode_Ticking;
		return;
	}

	// Not ticking
	if( mSwitches_byStuckVotes )
	// A switch wishes to stick
	{
		// Prepare the long interval
		mSwitches_wTickInterval = SWITCHES_LONG_DELAY;

		// Initialize the next wakeup
		TACCR1 = TAR + SWITCHES_LONG_DELAY;

		// Enable TACCR1 interrupt
		TACCTL1 |= (CCIE);
		
		// Transition to the stuck mode
		mSwitches_pfnOperatingMode = Switches_Mode_Stuck;
		return;
	}

	// Not ticking or stuck, so sleep
	// Do nothing
}

void Switches_Mode_Ticking( void )
{
	if( mSwitches_byTickingVotes )
	// A switch wishes to tick
	{	// Do nothing
		return;
	}

	// Not ticking
	if( mSwitches_byStuckVotes )
	// A switch wishes to stick
	{
		// Prepare the long interval
		mSwitches_wTickInterval = SWITCHES_LONG_DELAY;

		// Re-initialize the next wakeup
		TACCR1 += (SWITCHES_LONG_DELAY-SWITCHES_TICK_DIVISOR);

		// Transition to the stuck mode
		mSwitches_pfnOperatingMode = Switches_Mode_Stuck;
		return;
	}

	// Not ticking or stuck, so sleep
	// Disable TACCR1 interrupts
	TACCTL1 &= ~(CCIE);
	
	// Transition to the sleeping mode
	mSwitches_pfnOperatingMode = Switches_Mode_Sleeping;
}

void Switches_Mode_Stuck( void )
{
	if( mSwitches_byTickingVotes )
	// A switch wishes to tick
	{
		// Prepare the short interval
		mSwitches_wTickInterval = SWITCHES_TICK_DIVISOR;

		// Re-initialize the next wakeup
		TACCR1 = TAR + SWITCHES_TICK_DIVISOR;

		// Transition to the ticking mode
		mSwitches_pfnOperatingMode = Switches_Mode_Ticking;
		return;
	}

	// Not ticking
	if( mSwitches_byStuckVotes )
	// A switch wishes to stick
	{	// Do nothing
		return;
	}

	// Not ticking or stuck, so sleep
	// Disable TACCR1 interrupts
	TACCTL1 &= ~(CCIE);
	
	// Transition to the sleeping mode
	mSwitches_pfnOperatingMode = Switches_Mode_Sleeping;
}

void Switches_Enter_WaitForLow( void )
{
	// Enable the switch pullup
	P1REN |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Vote for sleep
	mSwitches_byTickingVotes &= ~(mSwitches_ptCurrentSwitch->byVoteBit);
	
	// Update after voting
	mSwitches_pfnOperatingMode();
	
	// Prepare for falling edges
	P1IES |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Clear the interrupt flag
	P1IFG &= ~(mSwitches_ptCurrentSwitch->byPortPin);
	
	// Enable port interrupts
	P1IE |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Transition to the place-holder state
	mSwitches_ptCurrentSwitch->pfnCurrentState = Switches_State_WaitForLow;
}

void Switches_State_WaitForLow( void )
{
	// This state remains as a placeholder
	// The functionality is split between the P1.6 ISR and
	// the Switches_DetectedLow() function, below.
}

void Switches_DetectedLow( void )
{
	// Disable the switch pullup
	P1REN &= ~(mSwitches_ptCurrentSwitch->byPortPin);

	// Cancel further port interrupts
	P1IE &= ~(mSwitches_ptCurrentSwitch->byPortPin);	

	// Vote for ticking
	mSwitches_byTickingVotes |= (mSwitches_ptCurrentSwitch->byVoteBit);
	
	// Update after voting
	mSwitches_pfnOperatingMode();
	
	// Pin is low, delay a tick before validating
	Switches_Enter_Delay( Switches_State_CheckLow );
}

void Switches_State_CheckLow( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();

	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(mSwitches_ptCurrentSwitch->byPortPin);

	if( (byP1IN & (mSwitches_ptCurrentSwitch->byPortPin)) )
	// Pin is high, restart state chain
	{	Switches_Enter_WaitForLow();
		return;
	}
	
	// Pin is low, declare the event
	Switches_DeclareEvent();
	
	// And transition
	Switches_Enter_WaitForHigh_InitFailureCount();
}

void Switches_Enter_WaitForHigh_InitFailureCount( void )
{
	// Initialize the failure count for fresh entries
	mSwitches_ptCurrentSwitch->wFailedHighCount = SWITCHES_STUCK_COUNT;

	// Call through to the core entry logic
	Switches_Enter_WaitForHigh();
}

void Switches_Enter_WaitForHigh( void )
{
	mSwitches_ptCurrentSwitch->pfnCurrentState = Switches_State_WaitForHigh;
}

void Switches_State_WaitForHigh( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(mSwitches_ptCurrentSwitch->byPortPin);

	if( !(byP1IN & (mSwitches_ptCurrentSwitch->byPortPin)) )
	// Pin is low
	{	// Update failure count
		mSwitches_ptCurrentSwitch->wFailedHighCount--;
		if( mSwitches_ptCurrentSwitch->wFailedHighCount )
		// Count not expired, stay in this stream
		{	return;
		}

		// Count expired, handle as stuck
		Switches_DeclareStuckEvent();
		Switches_Enter_CheckStuck();
		return;
	}
	
	// Pin is high, delay a tick before validating
	Switches_Enter_Delay( Switches_State_CheckHigh );
}

void Switches_State_CheckHigh( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(mSwitches_ptCurrentSwitch->byPortPin);

	if( !(byP1IN & (mSwitches_ptCurrentSwitch->byPortPin)) )
	// Pin is low, revert to waiting for high
	{	Switches_Enter_WaitForHigh();
		return;
	}
	
	// Restart state chain
	Switches_Enter_WaitForLow();
}

void Switches_Enter_Delay( PFN_STATE_VOID_VOID pfnNextState )
{
	mSwitches_ptCurrentSwitch->pfnNextStateAfterDelay = pfnNextState;
	mSwitches_ptCurrentSwitch->pfnCurrentState = Switches_State_Delay;
}

void Switches_State_Delay( void )
{
	mSwitches_ptCurrentSwitch->pfnCurrentState = mSwitches_ptCurrentSwitch->pfnNextStateAfterDelay;
}

void Switches_Enter_CheckStuck( void )
{
	// Revoke vote for ticking
	mSwitches_byTickingVotes &= ~(mSwitches_ptCurrentSwitch->byVoteBit);
	// Vote for stuck
	mSwitches_byStuckVotes |= (mSwitches_ptCurrentSwitch->byVoteBit);
	
	// Update after voting
	mSwitches_pfnOperatingMode();

	mSwitches_ptCurrentSwitch->pfnCurrentState = Switches_State_CheckStuck;
}

void Switches_State_CheckStuck( void )
{	BYTE byP1IN;

	// Enable the switch pullup
	P1REN |= (mSwitches_ptCurrentSwitch->byPortPin);

	// Delay prior to sampling pin
	Switches_PrechargeDelay();
	
	// Read the port
	byP1IN = P1IN;
	
	// Disable the switch pullup
	P1REN &= ~(mSwitches_ptCurrentSwitch->byPortPin);

	if( !(byP1IN & (mSwitches_ptCurrentSwitch->byPortPin)) )
	// Pin is low, stay in this state
	{	Switches_Enter_CheckStuck();
		return;
	}
	
	// Pin is high, exit to the normal ticking stream
	Switches_Exit_CheckStuck();
}

void Switches_Exit_CheckStuck( void )
{
	// Vote for ticking
	mSwitches_byTickingVotes |= (mSwitches_ptCurrentSwitch->byVoteBit);
	// Revoke vote for stuck
	mSwitches_byStuckVotes &= ~(mSwitches_ptCurrentSwitch->byVoteBit);
	
	// Update after voting
	mSwitches_pfnOperatingMode();
	
	// Initialize the failure count for re-entries
	mSwitches_ptCurrentSwitch->wFailedHighCount = 1;

	// Transition
	Switches_Enter_Delay( Switches_State_CheckHigh );
}

#pragma diag_suppress=Go004
#pragma optimize=size no_inline
void Switches_PrechargeDelay( void )
{
	// Delay a fixed number of cycles
	__delay_cycles( SWITCH_PULLUP_PRECHARGE_CYCLES );
}

#pragma vector=TIMERA1_VECTOR
__interrupt void TimerA3_ISR( void )
// Shared interrupt vector for TAR, CCR1 and CCR2
{	
	switch ( __even_in_range(TAIV, 10) )
	{
		case  2:
		// TACCR1
		// This ISR must not interrupt Port1 nor be interrupted by it.
			TACCR1 += (mSwitches_wTickInterval);
			mSwitches_ptCurrentSwitch = &mSwitches_tSW1;
			mSwitches_ptCurrentSwitch->pfnCurrentState();
			mSwitches_ptCurrentSwitch = &mSwitches_tSW2;
			mSwitches_ptCurrentSwitch->pfnCurrentState();
			break;
		
		case  4:
		// TACCR2
			break;
			
		case 10:
		// TAIFG
			break;
	}
}

#pragma vector=PORT1_VECTOR
__interrupt void Port1_ISR( void )
// Shared interrupt vector for all Port1 pins
// This ISR must not interrupt TACCR1 nor be interrupted by it.
{
	// Test whether P1.6 was triggered
	if( P1IE & BIT6 )
	// See if this interrupt is a possible source
	{
		if( P1IFG & BIT6 )
		// P1.6 experienced an interrupt
		{		
			// Take action
			mSwitches_ptCurrentSwitch = &mSwitches_tSW1;
			Switches_DetectedLow();
			return;
		}
	}
	
	// Test whether P1.7 was triggered
	if( P1IE & BIT7 )
	// See if this interrupt is a possible source
	{
		if( P1IFG & BIT7 )
		// P1.7 experienced an interrupt
		{		
			// Take action
			mSwitches_ptCurrentSwitch = &mSwitches_tSW2;
			Switches_DetectedLow();
			return;
		}
	}
}


