#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "TimerB3_UART.h"

#define RX_BUFFER_SIZE				0x0020

static __no_init PFN_STATE_VOID_VOID mTimerB3_UART_pfnRxState;
static __no_init BYTE mTimerB3_UART_abyRxBuffer[RX_BUFFER_SIZE];
static __no_init BYTE mTimerB3_UART_byIndexReadRx;
static __no_init BYTE mTimerB3_UART_byIndexWriteRx;
static __no_init BYTE volatile mTimerB3_UART_byQueueCountRx;
static __no_init BYTE mTimerB3_UART_byBitIndexRx;
static __no_init BYTE mTimerB3_UART_byAccumulatorRx;
#pragma diag_suppress=Pe177
static __no_init BYTE mTimerB3_UART_byUnused;
#pragma diag_default=Pe177

#define RX_BIT_ARRAY_SIZE				8
#define RX_START_BIT_HALF_INTERVAL		6
#define RX_STOP_BIT_INTERVAL			14
static const BYTE mTimerB3_UART_abyRxClocks[RX_BIT_ARRAY_SIZE] =
{	13,		// Bit0
	14,		// Bit1
	14,		// Bit2
	13,		// Bit3
	14,		// Bit4
	14,		// Bit5
	13,		// Bit6
	14,		// Bit7
};

// Rx State Functions
void TimerB3_UART_StateRx_WaitForStartEdge( void );
void TimerB3_UART_StateRx_SampleStartBit( void );
void TimerB3_UART_StateRx_AccumulateBits( void );
void TimerB3_UART_StateRx_SampleStopBit( void );

// Rx Helper Functions
void TimerB3_UART_EnterRx_WaitForStartEdge( void );
void TimerB3_UART_EnterRx_SampleStartBit( void );
void TimerB3_UART_EnterRx_AccumulateBits( void );
void TimerB3_UART_EnterRx_SampleStopBit( void );
void TimerB3_UART_DeclareRx( void );

#define TX_BUFFER_SIZE				0x0020

static __no_init PFN_STATE_VOID_VOID mTimerB3_UART_pfnTxState;
static __no_init BYTE mTimerB3_UART_abyTxBuffer[TX_BUFFER_SIZE];
static __no_init BYTE mTimerB3_UART_byIndexReadTx;
static __no_init BYTE mTimerB3_UART_byIndexWriteTx;
static __no_init BYTE mTimerB3_UART_byQueueCountTx;
static __no_init BYTE mTimerB3_UART_byBitIndexTx;
static __no_init WORD mTimerB3_UART_wDeaccumulatorTx;

#define TX_BIT_ARRAY_SIZE			10
static const BYTE mTimerB3_UART_abyTxClocks[TX_BIT_ARRAY_SIZE] =
{	14,		// Start
	13,		// Bit0
	14,		// Bit1
	14,		// Bit2
	13,		// Bit3
	14,		// Bit4
	14,		// Bit5
	13,		// Bit6
	14,		// Bit7
	25,		// Stop bits - 2
};

// Tx State Functions
void TimerB3_UART_StateTx_Idle( void );
void TimerB3_UART_StateTx_SendBits( void );

// Tx Helper Functions
void TimerB3_UART_EnterTx_Idle( void );
void TimerB3_UART_EnterTx_SendBits( void );
void TimerB3_UART_InnerQueueTxChar( BYTE byTx );

void TimerB3_UART_InitializeHW( void )
{
	// Drive TxD OUT high now
	TBCCTL2 = (CCIS_0|SCS|OUTMOD_0|OUT);

	// Configure TxD as a module output and RxD as a module input
	P4SEL |= (BIT2|BIT1);
}

void TimerB3_UART_InitializeApp( void )
{
	// Initialize Rx Queue
	mTimerB3_UART_byIndexReadRx = 0;
	mTimerB3_UART_byIndexWriteRx = 0;
	mTimerB3_UART_byQueueCountRx = 0;

	TimerB3_UART_EnterRx_WaitForStartEdge();

	// Initialize Tx Queue
	mTimerB3_UART_byIndexReadTx = 0;
	mTimerB3_UART_byIndexWriteTx = 0;
	mTimerB3_UART_byQueueCountTx = 0;
	
	TimerB3_UART_EnterTx_Idle();
}

// Rx Queue Functions
WORD TimerB3_UART_GetCountInRxQueue( void )
// Thread-safe
{
	return mTimerB3_UART_byQueueCountRx;
}

WORD TimerB3_UART_GetNextRxValue( void )
// Thread-safe for single clients
// If multiple clients might access, add thread-safe protection
// in the form of interrupt state wrappers (see below)
{	WORD wResult;

	// Start thread-safe wrapper here if needed
	if( 0 == mTimerB3_UART_byQueueCountRx )
	// No values exist
	{	wResult = NO_RX_AVAILABLE;
	}
	else
	// At least one value exists in the receive queue
	{
		// Get the next value
		wResult = mTimerB3_UART_abyRxBuffer[mTimerB3_UART_byIndexReadRx++];

		// Update the read index
		if( mTimerB3_UART_byIndexReadRx >= RX_BUFFER_SIZE )
		{	mTimerB3_UART_byIndexReadRx = 0;
		}
	
		// Update queue count
		mTimerB3_UART_byQueueCountRx--;
	}
	// End thread-safe wrapper here if needed
	
	return wResult;
}

// Rx State Functions
void TimerB3_UART_EnterRx_WaitForStartEdge( void )
// Prepare TBCCR1 to wait for an incoming start bit
{
	// Turn off TBCCR1 interrupts
	TBCCTL1 &= ~(CCIE|CCIFG);
	
	// Configure TBCCTL1 in capture mode to interrupt from a falling edge
	TBCCTL1 = (CM_2|CCIS_0|SCS|CAP);
	
	// Transition	
	mTimerB3_UART_pfnRxState = TimerB3_UART_StateRx_WaitForStartEdge;

	// Enable interrupts
	TBCCTL1 |= CCIE;
}

void TimerB3_UART_StateRx_WaitForStartEdge( void )
{
	TimerB3_UART_EnterRx_SampleStartBit();
}

void TimerB3_UART_EnterRx_SampleStartBit( void )
{
	// Prepare TBCCR1 to interrupt a half-bit later
	TBCCR1 += RX_START_BIT_HALF_INTERVAL;
	
	// Configure the compare mode
	TBCCTL1 = (CCIS_0|SCS);
	
	// Transition	
	mTimerB3_UART_pfnRxState = TimerB3_UART_StateRx_SampleStartBit;

	// Enable interrupts
	TBCCTL1 |= (CCIE);
}

void TimerB3_UART_StateRx_SampleStartBit( void )
{
	// Examine the sample
	if( TBCCTL1 & CCI )
	// Start bit is high
	{	// Abandon this value
		TimerB3_UART_EnterRx_WaitForStartEdge();
		return;
	}
	
	// Start bit is low, continue to the data bits
	TimerB3_UART_EnterRx_AccumulateBits();
}

void TimerB3_UART_EnterRx_AccumulateBits( void )
{
	// Update the interval for the first bit
	TBCCR1 += mTimerB3_UART_abyRxClocks[0];
	
	// Prepare the bit index
	mTimerB3_UART_byBitIndexRx = 1;

	// Transition
	mTimerB3_UART_pfnRxState = TimerB3_UART_StateRx_AccumulateBits;
}

void TimerB3_UART_StateRx_AccumulateBits( void )
// A data bit has been sampled
{
	// Shift the accumulator
	mTimerB3_UART_byAccumulatorRx >>= 1;
	// Save the bit
	if( TBCCTL1 & CCI )
	// Incoming bit is high
	{	mTimerB3_UART_byAccumulatorRx |= BIT7;
	}
	
	// Check whether done with this value
	if( mTimerB3_UART_byBitIndexRx >= 8 )
	{	TimerB3_UART_EnterRx_SampleStopBit();
		return;
	}
	
	// Prepare the next interval
	TBCCR1 += mTimerB3_UART_abyRxClocks[mTimerB3_UART_byBitIndexRx++];
}

void TimerB3_UART_EnterRx_SampleStopBit( void )
{
	// Prepare the next interval
	TBCCR1 += RX_STOP_BIT_INTERVAL;

	// Transition
	mTimerB3_UART_pfnRxState = TimerB3_UART_StateRx_SampleStopBit;
}

void TimerB3_UART_StateRx_SampleStopBit( void )
{
	// Transition regardless
	TimerB3_UART_EnterRx_WaitForStartEdge();

	if( 0 == (TBCCTL1 & CCI) )
	// Incoming bit is low, reject this value
	{	return;
	}
	
	// Declare a received value
	TimerB3_UART_DeclareRx();
}

void TimerB3_UART_DeclareRx( void )
// This function executes in the context of the receiver thread
{
	// Store the value on the queue
	mTimerB3_UART_abyRxBuffer[mTimerB3_UART_byIndexWriteRx++] =
		mTimerB3_UART_byAccumulatorRx;

	// Update the write index
	if( mTimerB3_UART_byIndexWriteRx >= RX_BUFFER_SIZE )
	{	mTimerB3_UART_byIndexWriteRx = 0;
	}

	// Update queue count
	mTimerB3_UART_byQueueCountRx++;

	// TODO:  Event-sink call here with the received value as a parameter
}

// Tx Queue Functions
void TimerB3_UART_InnerQueueTxChar( BYTE byTx )
// Write to the transmit queue
// Assumes the caller provides thread safety
{
	// Store the value on the queue
	mTimerB3_UART_abyTxBuffer[mTimerB3_UART_byIndexWriteTx++] = byTx;

	// Update the write index
	if( mTimerB3_UART_byIndexWriteTx >= TX_BUFFER_SIZE )
	{	mTimerB3_UART_byIndexWriteTx = 0;
	}

	// Update queue count
	mTimerB3_UART_byQueueCountTx++;
}

void TimerB3_UART_QueueTxChar( BYTE byTx )
// Does NOT check for overflow
{	
	// Block other threads from writing the queue
	istate_t isInterruptState = __get_interrupt_state();
	__disable_interrupt();

	// Call the inner queuing function
	TimerB3_UART_InnerQueueTxChar( byTx );
	
	// Unblock access
	__set_interrupt_state( isInterruptState );
	
	// Launch transmitter
	TimerB3_UART_EnterTx_SendBits();
}

void TimerB3_UART_QueueTxString( PSZ pszTxString )
{	WORD wIndex;
	BYTE byOut;

	// Block other threads from writing the queue
	istate_t isInterruptState = __get_interrupt_state();
	__disable_interrupt();
	
	for( wIndex = 0;; wIndex++ )
	{	byOut = pszTxString[wIndex];
		if( 0x00 == byOut )
		{	break;
		}
		
		// Call the inner queuing function
		TimerB3_UART_InnerQueueTxChar( byOut );
	}

	// Unblock access
	__set_interrupt_state( isInterruptState );

	// Launch transmitter
	TimerB3_UART_EnterTx_SendBits();
}

// Tx State Functions
void TimerB3_UART_EnterTx_Idle( void )
{
	// Disable TBCCR2
	TBCCTL2 &= ~(CCIE);

	mTimerB3_UART_pfnTxState = TimerB3_UART_StateTx_Idle;
}

void TimerB3_UART_StateTx_Idle( void )
{
}

void TimerB3_UART_EnterTx_SendBits( void )
{
	if( TimerB3_UART_StateTx_Idle != mTimerB3_UART_pfnTxState )
	// Transmitter already running, no need to restart
	{	return;
	}

	// Check whether any Tx values are in the queue
	if( 0 == mTimerB3_UART_byQueueCountTx )
	// Nothing to send
	{	TimerB3_UART_EnterTx_Idle();
		return;
	}

	// Prepare the bit index
	mTimerB3_UART_byBitIndexTx = 0;
	
	// Turn off TBCCR2 interrupt and clear flag
	TBCCTL2 &= ~(CCIE|CCIFG);
	
	// Prepare TBCCR2 in compare mode to interrupt two ACLKs hence
	TBCCR2 = TBR + 2;
	
	// Reset the output pin later as the start bit
	TBCCTL2 = (CCIS_0|SCS|OUTMOD_5|OUT);
	
	// Transition
	mTimerB3_UART_pfnTxState = TimerB3_UART_StateTx_SendBits;

	// Enable the interrupt
	TBCCTL2 |= CCIE;
}

void TimerB3_UART_StateTx_SendBits( void )
{
	if( mTimerB3_UART_byBitIndexTx >= TX_BIT_ARRAY_SIZE )
	// The stop bit just completed
	{	
		if( 0 == mTimerB3_UART_byQueueCountTx )
		// No more values remain
		{	
			// Shift to idle
			TimerB3_UART_EnterTx_Idle();
		}
		else
		// Restart for the next outgoing value
		{
			// Set to start 2 ACLKs later
			TBCCR2 = TBR + 2;
			
			// Continue to drive OUT high now, and clear the output pin later
			TBCCTL2 |= OUTMOD2;
			
			// Prepare the bit index to load the next value
			mTimerB3_UART_byBitIndexTx = 0;
		}
		return;
	}

	// Check whether we need to load the deaccumulator
	if( 0 == mTimerB3_UART_byBitIndexTx )
	// This bit interval is the start bit, so prepare the value
	{
		// Get the value
		mTimerB3_UART_wDeaccumulatorTx =
			mTimerB3_UART_abyTxBuffer[mTimerB3_UART_byIndexReadTx++];
	
		// Update the read index
		if( mTimerB3_UART_byIndexReadTx >= TX_BUFFER_SIZE )
		{	mTimerB3_UART_byIndexReadTx = 0;
		}
	
		// Update queue count
		mTimerB3_UART_byQueueCountTx--;
		
		// Ensure the high bits are set to emulate a stop bit
		mTimerB3_UART_wDeaccumulatorTx |= 0xFF00;
	}

	// The current bit is executing
	// Update the interval for this bit
	TBCCR2 += mTimerB3_UART_abyTxClocks[mTimerB3_UART_byBitIndexTx++];
	
	// and prep the next bit
	if( mTimerB3_UART_wDeaccumulatorTx & BIT0 )
	// Next bit is a 1
	{	TBCCTL2 &= ~(OUTMOD2);
	}
	else
	// Next bit is a 0
	{	TBCCTL2 |= (OUTMOD2);
	}
	
	// Update the deaccumulator
	mTimerB3_UART_wDeaccumulatorTx >>= 1;
}

#pragma vector=TIMERB1_VECTOR
__interrupt void TimerB3_ISR( void )
// Shared interrupt vector for TBR, CCR1 and CCR2
{	
	switch ( __even_in_range(TBIV, 10) )
	{
		case  2:
		// TBCCR1
			mTimerB3_UART_pfnRxState();
			break;
		
		case  4:
		// TBCCR2
			mTimerB3_UART_pfnTxState();
			break;
			
		case 14:
		// TBIFG
			break;
	}
}
