#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "UCA0_UART.h"

#define RX_BUFFER_SIZE				0x0020

static __no_init PFN_STATE_VOID_VOID mUCA0_UART_pfnRxState;
static __no_init BYTE mUCA0_UART_abyRxBuffer[RX_BUFFER_SIZE];
static __no_init BYTE mUCA0_UART_byIndexReadRx;
static __no_init BYTE mUCA0_UART_byIndexWriteRx;
static __no_init BYTE volatile mUCA0_UART_byQueueCountRx;
#pragma diag_suppress=Pe177
static __no_init BYTE mUCA0_UART_byUnused;
#pragma diag_default=Pe177

// Rx State Functions
void UCA0_UART_StateRx_DefaultRx( void );

// Rx Helper Functions
void UCA0_UART_EnterRx_DefaultRx( void );
void UCA0_UART_DeclareRx( void );

#define TX_BUFFER_SIZE				0x0020

static __no_init PFN_STATE_VOID_VOID mUCA0_UART_pfnTxState;
static __no_init BYTE mUCA0_UART_abyTxBuffer[TX_BUFFER_SIZE];
static __no_init BYTE mUCA0_UART_byIndexReadTx;
static __no_init BYTE mUCA0_UART_byIndexWriteTx;
static __no_init BYTE mUCA0_UART_byQueueCountTx;

// Tx State Functions
void UCA0_UART_StateTx_Idle( void );
void UCA0_UART_StateTx_SendBits( void );

// Tx Helper Functions
void UCA0_UART_EnterTx_Idle( void );
void UCA0_UART_EnterTx_SendBits( void );
void UCA0_UART_InnerQueueTxChar( BYTE byTx );

void UCA0_UART_InitializeHW( void )
{
	// Set UCSWRST
	UCA0CTL1 |= UCSWRST;
	
	// Initialize registers for 2400-8-N-1
	UCA0CTL0 = 0x00;
	UCA0CTL1 = (UCSSEL0|UCSWRST);
	UCA0BR0 = 0x0D;
	UCA0BR1 = 0x00;
	UCA0MCTL = 0x0C;
	UCA0STAT = 0x00;

	// Initialize flags
	IFG2 &= ~(UCA0RXIFG);
	IFG2 |= (UCA0TXIFG);

	// Clear UCSWRST
	UCA0CTL1 &= ~UCSWRST;

	// Enable Rx interrupt
	IE2 |= UCA0RXIE;
}

void UCA0_UART_InitializeApp( void )
{
	// Initialize Rx Queue
	mUCA0_UART_byIndexReadRx = 0;
	mUCA0_UART_byIndexWriteRx = 0;
	mUCA0_UART_byQueueCountRx = 0;

	UCA0_UART_EnterRx_DefaultRx();

	// Initialize Tx Queue
	mUCA0_UART_byIndexReadTx = 0;
	mUCA0_UART_byIndexWriteTx = 0;
	mUCA0_UART_byQueueCountTx = 0;
	
	UCA0_UART_EnterTx_Idle();
}

// Rx Queue Functions
WORD UCA0_UART_GetCountInRxQueue( void )
// Thread-safe
{
	return mUCA0_UART_byQueueCountRx;
}

WORD UCA0_UART_GetNextRxValue( void )
// Thread-safe for single clients
// If multiple clients might access, add thread-safe protection
// in the form of interrupt state wrappers (see below)
{	WORD wResult;

	// Start thread-safe wrapper here if needed
	if( 0 == mUCA0_UART_byQueueCountRx )
	// No values exist
	{	wResult = NO_RX_AVAILABLE;
	}
	else
	// At least one value exists in the receive queue
	{
		// Get the next value
		wResult = mUCA0_UART_abyRxBuffer[mUCA0_UART_byIndexReadRx++];

		// Update the read index
		if( mUCA0_UART_byIndexReadRx >= RX_BUFFER_SIZE )
		{	mUCA0_UART_byIndexReadRx = 0;
		}
	
		// Update queue count
		mUCA0_UART_byQueueCountRx--;
	}
	// End thread-safe wrapper here if needed
	
	return wResult;
}

// Rx State Functions
void UCA0_UART_EnterRx_DefaultRx( void )
{
	mUCA0_UART_pfnRxState = UCA0_UART_StateRx_DefaultRx;
}

void UCA0_UART_StateRx_DefaultRx( void )
// A value was just received
{
	// Declare it
	UCA0_UART_DeclareRx();
}

void UCA0_UART_DeclareRx( void )
// This function executes in the context of the receiver thread
{
	// Store the value on the queue
	mUCA0_UART_abyRxBuffer[mUCA0_UART_byIndexWriteRx++] = UCA0RXBUF;

	// Update the write index
	if( mUCA0_UART_byIndexWriteRx >= RX_BUFFER_SIZE )
	{	mUCA0_UART_byIndexWriteRx = 0;
	}

	// Update queue count
	mUCA0_UART_byQueueCountRx++;

	// TODO:  Event-sink call here with the received value as a parameter
}

// Tx Queue Functions
void UCA0_UART_InnerQueueTxChar( BYTE byTx )
// Write to the transmit queue
// Assumes the caller provides thread safety
{
	// Store the value on the queue
	mUCA0_UART_abyTxBuffer[mUCA0_UART_byIndexWriteTx++] = byTx;

	// Update the write index
	if( mUCA0_UART_byIndexWriteTx >= TX_BUFFER_SIZE )
	{	mUCA0_UART_byIndexWriteTx = 0;
	}

	// Update queue count
	mUCA0_UART_byQueueCountTx++;
}

void UCA0_UART_QueueTxChar( BYTE byTx )
// Does NOT check for overflow
{	
	// Block other threads from writing the queue
	istate_t isInterruptState = __get_interrupt_state();
	__disable_interrupt();

	// Call the inner queuing function
	UCA0_UART_InnerQueueTxChar( byTx );
	
	// Unblock access
	__set_interrupt_state( isInterruptState );
	
	// Launch transmitter
	UCA0_UART_EnterTx_SendBits();
}

void UCA0_UART_QueueTxString( PSZ pszTxString )
{	WORD wIndex;
	BYTE byOut;

	// Block other threads from writing the queue
	istate_t isInterruptState = __get_interrupt_state();
	__disable_interrupt();
	
	for( wIndex = 0;; wIndex++ )
	{	byOut = pszTxString[wIndex];
		if( 0x00 == byOut )
		{	break;
		}
		
		// Call the inner queuing function
		UCA0_UART_InnerQueueTxChar( byOut );
	}

	// Unblock access
	__set_interrupt_state( isInterruptState );

	// Launch transmitter
	UCA0_UART_EnterTx_SendBits();
}

// Tx State Functions
void UCA0_UART_EnterTx_Idle( void )
{
	// Disable the USCI_A0 transmit interrupt
	IE2 &= ~(UCA0TXIE);

	mUCA0_UART_pfnTxState = UCA0_UART_StateTx_Idle;
}

void UCA0_UART_StateTx_Idle( void )
{
}

void UCA0_UART_EnterTx_SendBits( void )
{
	if( UCA0_UART_StateTx_Idle != mUCA0_UART_pfnTxState )
	// Transmitter already running, no need to restart
	{	return;
	}

	// Check whether any Tx values are in the queue
	if( 0 == mUCA0_UART_byQueueCountTx )
	// Nothing to send
	{	UCA0_UART_EnterTx_Idle();
		return;
	}

	// Transition
	mUCA0_UART_pfnTxState = UCA0_UART_StateTx_SendBits;

	// Set the interrupt flag
	IFG2 |= UCA0TXIFG;
	
	// Enable the interrupt
	IE2 |= UCA0TXIE;
}

void UCA0_UART_StateTx_SendBits( void )
// The transmit register is empty
{
	if( 0 == mUCA0_UART_byQueueCountTx )
	// No more values remain in the buffer
	{	
		// Shift to idle
		UCA0_UART_EnterTx_Idle();
	}
	else
	// More values are available in the buffer
	{
		// Get the next value
		UCA0TXBUF = mUCA0_UART_abyTxBuffer[mUCA0_UART_byIndexReadTx++];
	
		// Update the read index
		if( mUCA0_UART_byIndexReadTx >= TX_BUFFER_SIZE )
		{	mUCA0_UART_byIndexReadTx = 0;
		}
	
		// Update queue count
		mUCA0_UART_byQueueCountTx--;
	}
}

#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI_TX_ISR( void )
{
// TODO:  If USCI_B0 ever shares this interrupt service routine, be sure to
// uncomment these conditional tests, and wrap the USCI_B0 handler similarly.
//	// Test the USCI_A0 interrupt enable
//	if( IE2 & UCA0TXIE )
//	// See if this interrupt is a possible source
//	{
//		if( IFG2 & UCA0TXIFG )
//		// USCI_A0 Tx interrupt flag is set
//		{
			mUCA0_UART_pfnTxState();
//			return;
//		}
//	}
}

#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI_RX_ISR( void )
{
// TODO:  If USCI_B0 ever shares this interrupt service routine, be sure to
// uncomment these conditional tests, and wrap the USCI_B0 handler similarly.
//	// Test the USCI_A0 interrupt enable
//	if( IE2 & UCA0RXIE )
//	// See if this interrupt is a possible source
//	{
//		if( IFG2 & UCA0RXIFG )
//		// USCI_A0 Rx interrupt flag is set
//		{
			mUCA0_UART_pfnRxState();
//			return;
//		}
//	}
}

